
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/wait.h>

#include "interface.h"
#include "support.h"

#if 0	/* 1: debug on;  0: debug off */
#  define DBG(cmd)	cmd
#else
#  define DBG(cmd)
#endif

#define MOUNT	"mount"
#define UMOUNT	"umount"

GtkWidget *progress_window;
GtkWidget *error_dialog;

char  *progress_msg	= "<b>Připojuji disketu do %s</b>";
char  *error_msg	= "<b>Disketu se nepovedlo připojit</b>";
char  *command		= NULL, 
      *mountpoint	= NULL;
int   pid		= -1;
int   cmd_pipe[2]	= { -1, -1 };
int   umount		= 0;



void
on_child_exits (int sig)
{
	int err;
	char buf[1024] = {0};
	int buf_len = 0;
	
	DBG(printf("(%d) have SIDCHLD...\n", pid); fflush(stdout);)
	if (pid > 0) {
		/* TODO precteni dat od detatka */
		buf_len  = read(cmd_pipe[0], buf, 1023 * sizeof(char));
		buf[buf_len] = 0;
		DBG(printf("readed: %s\n", buf); fflush(stdout);)

		/* vyzvednuti zombika */
		DBG(printf("child %d is dead.\n", pid); fflush(stdout);)
		waitpid(pid, &err, 0);
		DBG(printf("returned: %d\n", err); fflush(stdout);)

		/* likvidace progress_window*/
		gtk_widget_destroy(progress_window);
		
		/* neco malo na konec */
		if (err == EXIT_SUCCESS) {
			DBG(printf("ok\n"); fflush(stdout);)
			gtk_main_quit();
			exit(EXIT_SUCCESS);
		} else {
			error_dialog = create_error_dialog ();
			gtk_label_set_markup(GTK_LABEL(lookup_widget(error_dialog,
							"main_label")),
					error_msg);
			gtk_label_set_text(GTK_LABEL(lookup_widget(error_dialog,
							"error_label")),
					buf);
			gtk_widget_show (error_dialog);
		}
	}
}

int
main (int argc, char *argv[])
{
	int opt;
	char *buf = NULL;

#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif

	gtk_set_locale ();
	gtk_init (&argc, &argv);

	add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");

	while ((opt = getopt(argc, argv, "m:p:e:o:u")) != -1) {
		switch (opt) {
			case 'm':
				mountpoint = strdup(optarg);
				break;
			case 'p':
				progress_msg = strdup(optarg);
				break;
			case 'e':
				error_msg = strdup(optarg);
				break;
			case 'o':
				command = strdup(optarg);
				break;
			case 'u':
				umount = 1;
				break;
			case 'h':
			case '?':
				printf("Usage: %s -m mountpoint [-p progress_message] "
						"[-e error_message]\n\t\t[-o open_command] "
						"[-u]\n",
						argv[0]);
				exit(EXIT_FAILURE);
				break;
		}
	}
	if (!mountpoint) {
		fprintf(stderr, "%s: mountpoint must be set.\n", argv[0]);
		exit(EXIT_FAILURE);
	}
	
	progress_window = create_progress_window ();
	buf = (char *)malloc(strlen(progress_msg) + strlen(mountpoint) + 2);
	sprintf(buf, progress_msg, mountpoint);
	gtk_label_set_markup(GTK_LABEL(lookup_widget(progress_window, "main_label")),
			buf);
	free(buf);
	gtk_widget_show (progress_window);
	signal(SIGCHLD, on_child_exits);

	if (pipe(cmd_pipe) != 0) {
		perror(argv[0]);
		exit(EXIT_FAILURE);
	}
	fcntl(cmd_pipe[0], F_SETFL, O_NONBLOCK | O_ASYNC);
	fcntl(cmd_pipe[1], F_SETFL, O_NONBLOCK | O_ASYNC);

	pid = fork();
	usleep(10000);
	DBG(printf("fork to %d\n", pid); fflush(stdout);)
	switch (pid) {
		case 0: /* child */
			signal(SIGCHLD, SIG_DFL);
			DBG(printf("(%d) Mounting %s...", pid, mountpoint); fflush(stdout);)
			close(2);
			dup2(cmd_pipe[1], 2);
			close(cmd_pipe[0]);
			close(cmd_pipe[1]);
			if (execlp(umount ? UMOUNT : MOUNT, umount ? UMOUNT : MOUNT,
						mountpoint, NULL) == -1)
			{
				perror(MOUNT);
				exit(EXIT_FAILURE);
			}
			break;

		case -1: /* error */
			perror("fork");
			exit(EXIT_FAILURE);
			break;
			
		default: /* parent */
			close(cmd_pipe[1]);
			gtk_main();
			return EXIT_SUCCESS;
	}

	return EXIT_SUCCESS;

}

/* vim:set encoding=utf8: */
